;
; RAW DISK DUMP UTILITY FOR C1P/UK101/C2PDF/C4PMF/C8P/C3-OEM
;
; DUMPS THE CONTENTS OF A DISK AND SENDS OUTPUT
; TO THE SERIAL PORT AS ASCII HEX. IT WILL
; CAPTURE ALL DATA ON THE DISK INCLUDING THE
; TRACK/SECTOR FORMATTING INFORMATION.
;
; DUMPS EVERY BYTE ENCOUNTERED ON A TRACK
; STARTING AT THE INDEX PULSE.
;
; NOTE: DOS FORMATTED TRACKS WILL CONTAIN
; SPURIOUS CHARACTERS BEFORE AND AFTER SECTORS
; AND CAN BE IGNORED.
;
; NOTE 65U DISKS SWITCH ENCODING FROM 8E1 TO 8N1
; ON TRACKS 2-77 AFTER TRACK IDENTIFIER

;BASED ON ED'S OSIDUMP UTILITY
;UPDATED FOR OS65U, XMODEM ETC
;FREE FOR ALL USES MARK SPANKUS
;

ORG	=	$0300

MAXTRK5	=	40	; # TRACKS TO READ 5 1/4 disk
MAXPAG5	=	9	; # PAGES TO READ  5 1/4 disk
MAXTRK8 =   77	; # Tracks to read 8 disk
MAXPAG8 =   15	; # Pages to read 8 disk
FILLCH	=	$00	; FILL CHARACTER

; defined at end of program BUFFER	=	$1000	; TRACK BUFFER (to $2000 8")
DDPIA	=	$C000	; Disk controller PIA
DDACIA  =   $C010   ; Disk Controller Serial Port
C2ACIA  =   $FC00   ; ACIA on C2/C4/C8/C3
C1ACIA  =   $F000   ; ACIA on C1

;Control Character Constants XMODEM etc.
CR		=	$0D
LF		=	$0A
SOH		=	$01		; start block marker
STOX    =   $02     ; start of 1K block marker (STX is reserved)
EOT		=	$04		; end of text marker
ACK		=	$06		; good block acknowledged
NAK		=	$15		; bad block acknowledged
CAN		=	$18		; cancel (not standard, not supported)
ESC		=	$1B		; ESC to exit


;
; XMODEM storage
;
;
xmdata      =   $10
lastblk		=	xmdata			; flag for last block
blkno		=	xmdata+1		; block number
Xerrcnt		=	xmdata+2		; error counter 10 is the limit
crc			=	xmdata+3		; CRC lo byte  (two byte variable)
crch		=	xmdata+4		; CRC hi byte
ptr			=	xmdata+5		; data pointer (two byte variable)
ptrh		=	xmdata+6		;   "    "
eofp		=	xmdata+7		; end of file address pointer (2 bytes)
eofph		=	xmdata+8		;  "	"	"	"
Xretry		=	xmdata+9		; retry counter
Xretry2		=	xmdata+10		; 2nd counter
lastdata    =   xmdata+11       ;flag buffer of file
Xsavem      =   xmdata+12       ;saved index to resume filling send buffer

;xmodem status return values
WRONGBLK = $FD		;xmode recv wrong blk #
MISMATCH = $FC
ABORTRECV = $FE	    ;xmode recv abort recv

;storage space (anywhere, so why not page zero?)
STORE   =   $20   ;storage
MAXTRK  =   STORE+01 ;1 - 40 for 5.25, 77 for 8, 255 for HD
MAXTRH  =   STORE+02 ;1 - high byte max track for HD, 0 for floppy
MAXPAG  =   STORE+03 ;1 - 9 for 5.25, 15 for 8, 16 for HD
ACIACTL =   STORE+04 ;1 controls cassette/rs-232 ACIA divisor, format
DRVNUM  =   STORE+05 ;1 drive# 0-3
TRK     =   STORE+06 ;1 current track #
TRKHI   =   STORE+07 ;1 high byte current track # (block #)
CNT	    =   STORE+09 ;2 # bytes sent (unused)
TMP     =   STORE+10 ;1
PDEC8	=   STORE+11 ;2 word - hex to decimal source
PDEC9	=   STORE+13 ;1 flag byte
PDEC10	=   STORE+14 ;3 byte result hex to decimal
USEXMOD =   STORE+17 ;1 - use xmodem 0x80=CRC 0xC0=Checksum  else HEX
MACHINE =   STORE+18 ;1 C1/C2/C3 flag 00=C2/4/8, $40=C3Ser, $80=C1
OS65U   =   STORE+19 ;1 0x80 when 8" reading OS65U 0x40 when not, 0 when undetermined
                     ;0x80 = force OS65U  0x40 = force OS65D  0 = automatic


Rbuff		=	$40 ; to $C5 address of 133 byte xmodem buffer

;zero page storage
DRVACIA =   $E0	; 1 byte storage for current serial word format for DISK ACIA 8E1/8N1
TRKCNT  =   $E1 ; 1 bytes
PTRSTOR =   $E2 ; 2 bytes
TDATA   =   $E4 ; 2 bytes

	*=	ORG
	.EXE *      ;A65 emit OSI .lod start address operation
	SEI
	CLD
	LDA #$4C	;set C1P Warmstart
	STA $00
	LDA #<ORG
	STA $01
	LDA #>ORG
	STA $02
	LDA	$FE01	;determine machine type
	BEQ	SERTYP	;Is this serial system?
	LDA	#$00
	BIT	$DF00	;okay check C1/C2-C4
	BMI	C1TYPE
C1TYPE=*+1
	BIT	$80A9	;
SERTYP=*+1
	BIT	$40A9
	STA	MACHINE	;bit 7 = C1, bit 6 = C3, none =C2/C4
	LDA	#$15    ;B1 = 8N2 /16 RTStxIRQ  rxIRQ ;$15 = 8N1 /16 RTSNOtxIRQ NOrxIRQ; $B5 = irqs on 8N1
	STA	ACIACTL
	JSR	INZACIA
	LDA	#$00
	STA	DRVNUM
	STA	USEXMOD
	STA OS65U
    BIT MACHINE
	BVC *+5
	JMP SETDRV8  ;serial systems use 8" by default
	JMP SETDRV5

TOP
	JSR	DRWMENU
	LDX	#$00
TOPMSG
	LDA	SCRNMSG,X
	BEQ	MENUL0
	STA	$D0C5,X	;write "on Serial" mesg to screen
	INX
	BNE	TOPMSG

MENUL0
	JSR INKEY
	CMP #$20
	BCC MENUL0
	CMP #$80
	BCS MENUL0
	JSR OUTPUT
	CMP #$31
	BNE *+5
	JMP RAWREAD
	CMP #$32
	BNE *+5
	JMP TOP  ; placeholder
	CMP #$33
	BNE *+5
	JMP SETDRV
	CMP #$34
	BNE *+5
	JMP DRVTYPE
	CMP #$35
	BNE *+5
	JMP ACIADIV
	CMP #$36
	BNE *+5
	JMP SETXFER
	CMP #$37
	BNE *+5
	JMP ($FFFC)	;Exit
	CMP #$38
	BNE *+5
	JMP FORCE
	LDA #'?
	JSR OUTPUT
	JMP TOP

DRVTYPE
	JSR PRINT
	.BYTE $D,$A,'Enter your disk drive type (8) inch or (5).25 inch? >',0
	JSR INKEY
	JSR OUTPUT
	CMP #$38
	BEQ SETDRV8
	CMP #$35
	BEQ SETDRV5
INPERR
	LDA #'?
	JSR OUTPUT
	JMP TOP

SETDRV5
	LDA #MAXTRK5
	STA MAXTRK
	LDA #MAXPAG5
	STA MAXPAG
	LDA #$40		;flag not OS65U (it's a 5" drive)
	STA OS65U
	JMP TOP

SETDRV8
	LDA #MAXTRK8
	STA MAXTRK
	LDA #MAXPAG8
	STA MAXPAG
	LDA #$00		;flag disk type unknown (could be OS65U)
	STA OS65U
	JMP TOP
FORCE
	LDA MAXTRK
	CMP #MAXTRK5
	BEQ FORCED
	BIT OS65U
	BMI FORCED
	BVC FORCEU
	LDA #$00
FORCEU=*+1
	BIT $80A9
FORCED=*+1
	BIT $40A9
	STA OS65U
NOFORCE
	JMP TOP


ACIADIV
	JSR PRINT
	.BYTE $D,$A,'Set (N)ormal or 16x (F)aster? >',0
	JSR INKEY
	JSR OUTPUT
	ORA #$20
	CMP #'f
	BEQ DIV1
	CMP #'n
	BEQ DIV16
	JMP INPERR

DIV1
	LDA	#$14  ;8N1 div 1

DIV16=*+1
	BIT $15A9  ;$15A9	 ;15=8N1 /16 -RTS active no irq
	STA ACIACTL
	JSR INZACIA
	JMP TOP

SETDRV
	JSR PRINT
	.BYTE $D,$A,'Select drive (A/B/C/D) ? >',0
	JSR INKEY
	JSR OUTPUT
	AND #$DF
	CMP #'A
	BCC SETERR
	CMP #'E
	BCS SETERR
	SBC #'@		;carry is clear so val -1
	STA DRVNUM
	JMP TOP
SETERR
	JMP	INPERR

SETXFER
	JSR PRINT
	.BYTE $D,$A,'Use (X)modem, Xmodem(C)RC or(H)ex transfer ? >',0
	JSR INKEY
	JSR OUTPUT
	AND #$DF
	CMP #'X
	BNE SETXF1
	LDA #$C0	;bit 7,6 set = Xmodem(Checksum)
	BNE SETXM
SETXF1
	CMP #'C
	BNE SETXF2
	LDA #$80
	BNE SETXM	;bit 7 set = Xmodem/CRC
SETXF2
	CMP #'H
	BNE SETERR ;hex
	LDA #$00
SETXM
	STA USEXMOD
SETTOP
	JMP TOP

;-----------------------------------------------------------
RAWREAD

	JSR	PRINT
	.BYTE	CR,LF
	.BYTE	'WRITE PROTECT ALL DISKS!',CR,LF
	.BYTE	CR,LF
	.BYTE	'INSERT DISK TO BE DUMPED',CR,LF
	.BYTE   'ESC KEY OR ^X ABORTS...',CR,LF,0

	JSR ANYKEY
	CMP #$1B
	BEQ SETTOP

	JSR INITPIA
	JSR SELDRV

	LDA #0
	STA TRK
	STA lastdata

	JSR TZERO	; STEP TO TRACK 0
	BCS NOTRK
	LDA #$58    ;0 10 110 00 ;$58-  recv irq, RTS HIGH no xmit irq,  8E1,  DIVIDE BY 1,
	STA DRVACIA

LOOP
	LDA TRK
	BNE LOOP3
	JSR RDTZERO
	BCC LOOP5
	JMP NODRIVE
LOOP3
	LDA TRK
	CMP MAXTRK
	BCS QUIT
	JSR READMSG
	JSR RDTRKU
	BCC LOOP5
	JMP NODRIVE
LOOP5
	LDA MAXPAG   ;how many pages to read?  09/0F
	JSR SAVBUF
	BCS QUIT
	JSR FRACIANW
	BCC LOOP4
	CMP #$1B   ;ESC key?
	BEQ QUIT
	CMP #$18   ;^X
	BEQ QUIT
LOOP4
	LDA TRK
	CMP MAXTRK
	BEQ QUIT
	JSR TNEXT	; STEP TO NEXT TRACK
LOOP2
	JMP LOOP

QUIT
	JSR TZERO
	JSR INITPIA
	JSR PRINT
	.BYTE CR,LF,CR,LF
	.BYTE ';DONE ... REMOVE DISK',CR,LF,0
	JSR ANYKEY
	JMP TOP


NOTRK
	JSR INITPIA
	JSR PRINT
	.BYTE CR,LF,CR,LF, 'ERROR Seeking Track 0',0
	JSR ANYKEY
	JMP TOP

NODRIVE
	JSR TZERO
	JSR INITPIA
	JSR PRINT
	.BYTE CR,LF, 'ERROR Waiting for disk index!',0
	JSR ANYKEY
	JMP TOP


; SAVE BUFFER CONTENTS (A) AS HEX

SAVBUF
	PHA
	JSR RESPTR
	PLA
	STA TRKCNT
	LDA #0
	STA CNT
	STA CNT+1
	TAY
	TAX
	LDA USEXMOD
	BNE SAVXMOD1
	BEQ SAVBUF5	 ;always

SAVBUF1
	CPX #0
	BNE SAVBUF2
	JSR CRLF

	JSR FRACIANW
	BCC SAVBUFX
	CMP #$1B   ;ESC key?
	BEQ SAVABORT
	CMP #$18   ;^X
	BNE SAVBUFX
SAVABORT
	JSR PRINT
	.BYTE $A,$D,'Abort!',0
	SEC
	RTS


SAVBUFX
	LDX #16
	BNE SAVBUF3

SAVBUF2
	LDA #' '
	JSR OUTPUT
SAVBUF3
	LDA (TDATA),Y
	JSR PHEXA
	INC CNT
	BNE SAVBUF4
	INC CNT+1
SAVBUF4
	DEX
	INY
	BNE SAVBUF1
	INC TDATA+1
	DEC TRKCNT
SAVBUF5
	LDA TRKCNT
	BNE SAVBUF1
	JSR CRLF
	CLC
	RTS


; SAVE BUFFER CONTENTS (A) pages USING XMODEM

SAVXMOD
	PHA
	JSR RESPTR
	PLA
	STA TRKCNT

SAVXMOD1

	LDA TDATA
	STA ptr
	LDA TDATA+1
	STA ptrh
	LDA MAXPAG
	CLC
	ADC TDATA+1
	STA eofph
	LDA TDATA
	STA eofp

	LDA TRK
	CLC
	ADC #$01
	CMP MAXTRK
	BNE SAVXMOD2
	LDA #$C0	 ;last block flag
	STA lastdata

SAVXMOD2
	JSR XResume
	BCC XMODBUF5
	JSR PRINT
	.BYTE $A,$D,'Error sending data!',0
	SEC
	RTS

XMODBUF5
	RTS



; INIT POINTERS, CLEAR BUFFER
INIT
	JSR RESPTR	; SET POINTERS
	LDY #0
	LDA #FILLCH
INIT1
	STA (TDATA),Y
	INY 
	BNE INIT1
	INC TDATA+1
	DEC TRKCNT
	BNE INIT1
	;fall through

; RESET BUFFER POINTERS
RESPTR
	LDA MAXPAG
	STA TRKCNT
	LDA #<BUFFER
	STA TDATA
	LDA #>BUFFER
	STA TDATA+1
	RTS

; INIT DISK CONTROLLER PIA
INITPIA
	LDY #0
	LDA #$40
	STY DDPIA+1	; SELECT DDRA
	STA DDPIA	; SET PORTA TO ALL INPUTS except PA6
	LDX #4		; 0000 0100
	STX DDPIA+1	; SELECT PORTA
	STA DDPIA    ; SET PB6 HIGH
	STY DDPIA+3	; SELECT DDRB
	DEY 
	STY DDPIA+2	; SET PORTB TO ALL OUTPUTS
	STX DDPIA+3	; SELECT PORTB
	STY DDPIA+2	; SET PORTB OUTPUTS HIGH
	RTS


;SELECT DRIVE PB5 PA6  DRIVE (1-4)
;              0   0	#4
;              0   1	#3
;              1   0	#2
;              1   1	#1

SELDRV
	LDA DRVNUM
	LSR A
	TAY 
	BCC *+5
	LDA #$00
	BIT $40A9
	STA DDPIA
	LDA DDPIA+2
	ORA #$20
	CPY #$01
	BCC *+4
	AND #$DF
	STA DDPIA+2
	RTS

; STEP TO TRACK 0

TZERO
	LDA MAXTRK	;max num tracks
	ADC #$08	;plus a few more
	STA TMP		;MAX times to step before abort
	LDA DDPIA+2
	AND #$FB	; 1111 1011
	BNE TZERO3	; DIR=INWARDS

TZERO1
	DEC TMP
	BNE TZERO5
	SEC
	RTS
TZERO5  
	LDA #2		; 0000 0010
	BIT DDPIA	; TEST 'TRK0' PIN
	BNE TZERO2	; AT TRACK 0
	LDA #0
	STA TRK
	CLC
	RTS

TZERO2
	LDA DDPIA+2	; DIR=OUTWARDS (TO TRK0)
	ORA #$04
TZERO3
	STA DDPIA+2	; SET 'DIR' PIN
	JSR DELAY2
	AND #$F7	; 1111 0111
	STA DDPIA+2	; SET 'STEP' PIN LOW
	JSR DELAY2
	ORA #8		; 0000 1000
	STA DDPIA+2	; SET 'STEP' PIN HIGH
	LDX #32 	; 40 MS
	JSR DELAY
	BEQ TZERO1	; ALWAYS

; STEP TO NEXT TRACK

TNEXT
	LDA DDPIA+2
	AND #$FB	; 1111 1011
				; DIR=INWARDS
	STA DDPIA+2	; SET 'DIR' PIN
	JSR DELAY2
	AND #$F7	; 1111 0111
	STA DDPIA+2	; SET 'STEP' LOW
	JSR DELAY2
	ORA #8		; 0000 1000
	STA DDPIA+2	; SET 'STEP' HIGH
	INC TRK
	LDX #32	    ; 40MS
	JMP DELAY


; READ CURRENT TRACK TO BUFFER
; RETURNS Y=index of last byte read

RDTZERO
	JSR INIT
	LDA DDPIA+2  ; PRESERVE DISK SEL BIT
	AND #$7F	; 0111 1111
	STA DDPIA+2	; LOAD DISK HEAD
	LDX #$FF	; 320MS
	JSR DELAY

	LDA #$80
	STA TMP
	LDX #$00

RDTRK1

	BIT DDPIA	; WAIT for INDEX PULSE
	BPL RDTRK8
	DEX 
	BNE RDTRK1
	DEC TMP
	BNE RDTRK1
	SEC
	RTS


RDTRK8
	LDX DRVACIA	; usually $58-DIVIDE BY 1, 8E1, RTS HIGH no IRQs
	LDA #3
	LDY #0
RDTRK2			; now past index hole, reset acia, prepare to read data
	BIT DDPIA	; WAIT END OF
	BPL RDTRK2	; INDEX PULSE
	STA DDACIA  ; RESET ACIA
	STX DDACIA  ; SET DISK SERIAL WORD FORMAT/CLEAR FLAGS

		; READ UNTIL ALL PAGES DONE

RDTRK3
	BIT DDPIA	; TEST FOR INDEX PULSE PRESENT AGAIN
	BPL RDTRK7
	LDA DDACIA ;test byte ready?
	LSR A
	BCC RDTRK3	  ;byte not ready?

RDTRK4
	LDA DDACIA+1 ; READ BYTE
RDTRK9
	STA (TDATA),Y
	INY
	BNE RDTRK3
	INC TDATA+1
	DEC TRKCNT
	BNE RDTRK3

RDTRK7
	LDA DDPIA+2
	ORA #$80
	STA	DDPIA+2	; UNLOAD DISK HEAD
	CLC
	RTS

; READ TRACK 1-76 TO BUFFER
; RETURNS Y=index of last byte read

RDTRKU
	JSR INIT
	LDA DDPIA+2  ; PRESERVE DISK SEL BIT
	AND #$7F	; 0111 1111
	STA DDPIA+2	; LOAD DISK HEAD
	LDX #$FF	; 320MS
	JSR DELAY

RDTRKU1
	BIT DDPIA	; WAIT START OF	INDEX PULSE
	BMI RDTRKU1
	PHA
	PLA
	PHA
	PLA
	PHA   ;delay
	PLA
	LDX DRVACIA	; usually $58-DIVIDE BY 1, 8E1, RTS HIGH no IRQs
	LDA #3
	LDY #0
    BIT OS65U
    BVS RDTRK2  ;can't be OS65U, we don't look for it

RDTRKU2 		; now past index hole, reset ACIA, prepare to read data
	BIT DDPIA	; WAIT END OF
	BPL RDTRKU2	; INDEX PULSE

	STA DDACIA  ; RESET ACIA
	STX DDACIA  ; SET DISK SERIAL WORD FORMAT/CLEAR FLAGS

	LDX #$00
RDTRKUP
	; READ 1st 3 bytes as 8E1 OS65U uses $80, tkLo, tkHi  OS65D uses $43, $57,Trk#,$58
	BIT DDPIA	; TEST FOR INDEX PULSE PRESENT AGAIN
	BPL RDTRKU5
	LDA DDACIA ;test byte ready?
	LSR A
	BCC RDTRKUP	  ;byte not ready?
	LDA DDACIA+1 ; READ BYTE
	STA (TDATA),Y
	CPX #$00
	BNE RDTYPE
	CMP #$43
	BNE *+3	  ;flag this is a 65D format 8E1 and continue reading there
	TAX
	CMP #$80
	BNE *+3
	TAX
RDTYPE
	INY
	CPY #$03
	BNE RDTRKUP
	BIT OS65U
	BMI RDELAY0  ;force OS65U read
	CPX #$80
	BNE RDTRK3   ;didn't find OS65U start of track marker so continue in current format

	; once we've read 1st 3 bytes at 8E1, reset ACIA & set 8N1
RDELAY0
	LDX #$07
RDELAY1
	DEX
	BNE RDELAY1	 ;delay same as 65u
	LDA #$03
	LDX #$54
	STA DDACIA		; RESET ACIA
	STX DDACIA		; SET DISK SERIAL WORD FORMAT/CLEAR FLAGS
	    
RDTRKU3
	BIT DDPIA		; TEST FOR INDEX PULSE PRESENT AGAIN
	BPL RDUCHK
	LDA DDACIA		;test byte ready?
	LSR A
	BCC RDTRKU3		;byte not ready?
	LDA DDACIA+1	; READ BYTE
	BMI RDTRKU3		; skip byte(s) with hi bit set	(noise)
	BPL RDTRKU8

RDTRKU4 			;finally read rest of track
	BIT DDPIA		; TEST FOR INDEX PULSE PRESENT AGAIN
	BPL RDUCHK
	LDA DDACIA 		;test byte ready?
	LSR A
	BCC RDTRKU4		;byte not ready?
	LDA DDACIA+1	; READ BYTE
RDTRKU8
	STA (TDATA),Y
	INY 
	BNE RDTRKU4
	INC TDATA+1
	DEC TRKCNT
	BNE RDTRKU4

RDUCHK
.IFN 0 <
	LDA TRK
	BEQ *+5
	JSR OSCHK
    BCC RDTRKU5
	JSR CHKERR
>
RDTRKU5			;now at end of track/index pulse start
	LDA	DDPIA+2
	ORA	#$80
	STA	DDPIA+2	;UNLOAD DISK HEAD
	CLC
	RTS
.IFN 0 <
OSCHK   ;check OS65U checksum  3 byte track header + $E00 data bytes
    JSR RESPTR
	LDA #$04
	STA PTRSTOR
	LDA #$0F	;1 higher than target #pages
	STA PTRSTOR+1
    LDA #$00
	TAX
	TAY
OSCHK1
    CLC
	ADC (TDATA),y
	BCC OSCHK2
	INX
	BEQ OSCHK2
OSCHK2
    INY
	BNE OSCHK3
	INC TDATA+1
OSCHK3
	DEC PTRSTOR
	BNE OSCHK1
	DEC PTRSTOR+1
	BNE OSCHK1
	INY
	CMP (TDATA),y
	BNE OSCHKF
	DEY
	TXA
	CMP (TDATA),Y
	BNE OSCHKF
OSCHKOK=*
	CLC
	RTS
OSCHKF
	SEC
	RTS
>
; DELAY 1.25MS PER LOOP

DELAY
	LDY	#$F8
DELAY1
	DEY
	BNE DELAY1
	EOR $FF,X
	DEX
	BNE DELAY
	RTS

DELAY2
	JSR DELAY21
DELAY21
	RTS


; PRINT IN-LINE STRING

PRINT
	PLA
	STA PTRSTOR
	PLA 
	STA PTRSTOR+1
	LDY #1
PRINT1  
	LDA (PTRSTOR),Y
	BEQ PRINT2
	JSR OUTPUT
	INY 
	BNE PRINT1
	INC PTRSTOR+1
	BNE PRINT1
PRINT2  
	TYA 
	SEC 
	ADC PTRSTOR
	STA PTRSTOR
	BCC PRINT3
	INC PTRSTOR+1
PRINT3
	CLC
	JMP (PTRSTOR)

; PRINT HEX WORD (A,X)

PHEX
	JSR PHEXA
	TXA

; PRINT HEX BYTE (A)
PHEXA
	PHA
	LSR A
	LSR A
	LSR A
	LSR A
	JSR PHEXA1
	PLA
PHEXA1
	AND #$F
	ORA #'0'
	CMP #'9'+1
	BMI PHEXA2
	CLC
	ADC #7
PHEXA2
	JMP OUTPUT

; PRINT DECIMAL (A)
PDECA
	TAX
	LDA	#0

; PRINT DECIMAL (A,X)

PDEC
	STX PDEC8
	STA PDEC8+1
	LDA #0
	LDX #2
PDEC1
	STA PDEC10,X
	DEX
	BPL PDEC1
	LDY #16
	PHP
	SEI
	SED
PDEC2
	ASL PDEC8
	ROL PDEC8+1
	LDX #2
PDEC3
	LDA PDEC10,X
	ADC PDEC10,X
	STA PDEC10,X
	DEX
	BPL PDEC3
	DEY
	BNE PDEC2
	PLP
	LDX #6
	LDA #0
	STA PDEC9
PDEC4
	LDY #3
	LDA #0
PDEC5
	ASL PDEC10+2
	ROL PDEC10+1
	ROL PDEC10
	ROL A
	DEY
	BPL PDEC5
	TAY
	BNE PDEC6
	CPX #1
	BEQ PDEC6
	LDY PDEC9
	BEQ PDEC7
PDEC6
    INC PDEC9
	ORA #$30
	JSR OUTPUT
PDEC7
    DEX
	BNE PDEC4
	RTS

CRLF
	LDA #CR
	JSR OUTPUT
	LDA #LF
	JMP OUTPUT

ANYKEY
	JSR PRINT
	.BYTE CR,LF
	.BYTE 'PRESS ANY KEY WHEN READY',CR,LF,0
	JMP INKEY

.IFN 0 <
CHKERR
	BIT USEXMOD
	BMI READMS1
	JSR PRINT
	.BYTE	' BAD CHECKSUM ',0
	RTS
>

READMSG
	BIT USEXMOD
	BMI READMS1 	;no mesg with xmodem
	PHA
	JSR	PRINT
	.BYTE CR,LF,CR,LF
	.BYTE ';READING TRACK ',0
	PLA
	JMP	PDECA
READMS1
	RTS				;no msg on C3

SCRNMSG
    .BYTE 'OSI DUMP ON SERIAL NOW',0

;Track 0 format
;10 millisecond delay after index hole
;0,1 load address of the track in hi,lo form
;2, page count of how much data is written on track 0, usually 8
;3+ data

;Track 1-n Format
;0,1 -a 2 byte start code $43, $57
;2,  -BCD track number
;3,  -track type code (always $58)

; sector data
;1,  -sector start code $76
;1,  -sector number in binary
;1   -sector length (#pages) in binary
;x   -sector data (1-8/12 pages)
;2   -end of sector mark $47, $53
; next sector

;track 1-76 format
; 8E1
; 0x80, tklo, tkhi, <reset ACIA to 8N1> 0x1F ... data ...


DRWMENU
	JSR FLUSHM
DRWMEN0
	 JSR PRINT
	.BYTE $D,$A,$D,$A
	.BYTE ' OSI Serial Disk Dump ',$D,$A
	.BYTE '----------------------',$D,$A
	.BYTE '1. Dump Raw Image',$D,$A
	.BYTE '3. Select Drive',$D,$A
	.BYTE '4. Set Drive Type',$D,$A
	.BYTE '5. Set Serial Divisor',$D,$A
	.BYTE '6. Xmodem/Hex xfer opts',$D,$A
	.BYTE '7. Exit              ',$D,$A
	.BYTE '                     ',$D,$A
	.BYTE 'Drv=',$0
	LDA DRVNUM
	CLC
	ADC #$41
	JSR	OUTPUT
	LDA	MAXPAG
	CMP #$0A
	BCS DRWMN1
	JSR PRINT
	.BYTE '/5.25',0
	BCC DRWMN2
DRWMN1
	JSR PRINT
	.BYTE '/8 ',0
DRWMN2
	JSR PRINT
	.BYTE ' Baud@1',0
   	LDA ACIACTL
	LSR A
	BCC DRWMN3
	JSR PRINT
	.BYTE 'x  ',0
	JMP DRWMN4
DRWMN3
	JSR PRINT
	.BYTE '6x ',0
	
DRWMN4
	BIT USEXMOD
	BPL DRWMN5
	BVC DRWMN8
	JSR PRINT
	.BYTE 'Xmdm ',0
	BCC DRWMN6
DRWMN8
	JSR PRINT
	.BYTE 'Xmdm/crc ',0
	BCC DRWMN6
DRWMN5	
	JSR PRINT
	.BYTE 'Hex ',0
DRWMN6
	LDA MAXTRK
	CMP #MAXTRK5
	BEQ DRWMN9
	BIT OS65U
	BPL DRWMN7
    JSR PRINT
	.BYTE 'force 65U',0
	BCC DRWMN9
DRWMN7
    BVC DRWMN9
	JSR PRINT
	.BYTE 'force 65D',0
DRWMN9
	JSR PRINT
	 .BYTE $D,$A,'--->',0
	RTS

INZACIA
	BIT MACHINE
	BMI INZAC1
    LDA #$03	;reset ACIA
	STA C2ACIA
	LDA ACIACTL	;
	STA C2ACIA
	RTS
INZAC1			;reset C1 ACIA
	LDA #$03
	STA C1ACIA
	LDA ACIACTL
	STA C1ACIA
	RTS

Get_Chr
FRACIANW  ; read from ACIA no wait	carry clear when no data
	BIT MACHINE
	BMI FRAC1NW
	LDA C2ACIA
	LSR A
	BCC FRAC1NW-1
	LDA C2ACIA+1
	RTS
FRAC1NW
	LDA C1ACIA
	LSR A
	BCC FRAC1NW-1
	LDA C1ACIA+1
	RTS

INKEY
FRACIA   	 ;read from ACIA carry set on abort return value in A
	BIT MACHINE
	BMI FRAC1
FRSER			; read from C3
	LDA C2ACIA
	LSR A
	BCC FRACIA
	LDA C2ACIA+1
	CLC
ACIARET
	RTS
FRAC1			;read from C1
	LDA C1ACIA
	LSR A
	BCC FRACIA
	LDA C1ACIA+1
	CLC
	RTS

OUTPUT
Put_Chr
TOACIA
	PHA
	BIT MACHINE
	BMI TOAC1
TOACIA1
	LDA C2ACIA   ;wait for TxEmpty
	LSR A
	LSR A
	BCC TOACIA1
	PLA
	STA C2ACIA+1
	RTS
TOAC1
	LDA C1ACIA
	LSR A
	LSR A
	BCC TOAC1
	PLA
	STA C1ACIA+1
	RTS


;----------------------------------------------------------------------------
; XMODEM/CRC Sender for the 65C02
;
; packet is 128 bytes, last packet is zero padded
; packet consists of <SOH=$01><blk# (starting at 1)><255-block#>< 128 data bytes><crclo><crc hi> for 133 bytes
; "C" is used to acknowledge & start transfer
;
; By Daryl Rictor Aug 2002
;
; A simple file transfer program to allow upload from the SBC to a
; console device utilizing the x-modem/CRC transfer protocol.  Requires just
; under 1k of either RAM or ROM, 132 bytes of RAM for the receive buffer,
; and 8 bytes of zero page RAM for variable storage.
;
;**************************************************************************
; This implementation of XMODEM/CRC does NOT conform strictly to the
; XMODEM protocol standard in that it (1) does not accurately time character
; reception or (2) fall back to the Checksum mode.
; (1) For timing, it uses a crude timing loop to provide approximate
; delays.  These have been calibrated against a 1MHz CPU clock.  I have
; found that CPU clock speed of up to 5MHz also work but may not in
; every case.  Windows HyperTerminal worked quite well at both speeds!
;
; (2) Most modern terminal programs support XMODEM/CRC which can detect a
; wider range of transmission errors so the fallback to the simple checksum
; calculation was not implemented to save space.
;**************************************************************************
; xNOT IN THIS PROGRAM
; xFiles uploaded via XMODEM-CRC will have the load address contained in
; xthe first two bytes in little-endian format:
; x FIRST BLOCK
; x    offset(0) = lo(load start address),
; x    offset(1) = hi(load start address)
; x    offset(2) = data byte (0)
; x    offset(n) = data byte (n-2)
;
; Subsequent blocks
;     offset(n) = data byte (n)
;
; One note, XMODEM send 128 byte blocks.  If the block of memory that
; you wish to save is smaller than the 128 byte block boundary, then
; the last block will be padded with zeros.  Upon reloading, the
; data will be written back to the original location.  In addition, the
; padded zeros WILL also be written into RAM, which could overwrite other
; data.
;
;-------------------------- The Code ----------------------------
;
; zero page variables (adjust these to suit your needs)
;
;
;xmdata      =   $35
;lastblk		=	xmdata			; flag for last block
;blkno		=	xmdata+1		; block number
;Xerrcnt		=	xmdata+2		; error counter 10 is the limit

;crc			=	xmdata+3		; CRC lo byte  (two byte variable)
;crch		=	xmdata+4		; CRC hi byte

;ptr			=	xmdata+5		; data pointer (two byte variable)
;ptrh		=	xmdata+6		;   "    "

;eofp		=	xmdata+7		; end of file address pointer (2 bytes)
;eofph		=	xmdata+8		;  "	"	"	"

;Xretry		=	xmdata+9		; retry counter
;Xretry2		=	xmdata+10		; 2nd counter
;lastdata     =   xmdata+11       ;flag buffer of file

;
;
; non-zero page variables and buffers
;
;
;Xbuff		=	$4000      	; temp 133 byte receive buffer
;					;(place anywhere, page aligned)
;
;
;  tables and constants
;
;
; The crclo & crchi labels are used to point to a lookup table to calculate
; the CRC for the 128 byte data blocks.  There are two implementations of these
; tables.  One is to use the tables included (defined towards the end of this
; file) and the other is to build them at run-time.  If building at run-time,
; then these two labels will need to be un-commented and declared in RAM.
;
;crclo		=	$7D00      	; Two 256-byte tables for quick lookup
;crchi		= 	$7E00      	; (should be page-aligned for speed)
;
;
;
; XMODEM Control Character Constants


;
;^^^^^^^^^^^^^^^^^^^^^^ Start of Program ^^^^^^^^^^^^^^^^^^^^^^
;
; Xmodem/CRC download routine
; By Daryl Rictor, August 8, 2002
;
; v0.1  code creation
;
;
;		*= 	$7800		; Start of program (adjust to your needs)
;
; Enter this routine with the beginning address stored in the zero page address
; pointed to by TDATA & TDATA+1 and the ending address stored in the zero page address
; pointed to by eofp & eofph.
;
;
XResume
	BIT lastdata
	BVC XModemStart	;bit 6 = 0, means its 1st block
	LDX Xsavem
	LDY #$00
	STY Xerrcnt
	;JSR GetByte
	JSR FlushQ		;MAC/Linux lrzsz sends us NAKs when we we're busy
	CMP #CAN
	BEQ XAbort
	CMP #ESC
	BEQ XAbort

	BNE	LdBuff1
XAbort
	JMP GotAbort

;XMODEM Send
XModemStart
	LDA #$40		;bit tst V = 1, means not 1st block next time
	STA lastdata
	JSR PrintMsg	; send prompt and info
	LDA #$00		;
	STA Xerrcnt		; error counter set to 0
	STA lastblk		; set flag to false
	LDA #$01		;
	STA blkno		; set block # to 1
Wait4CRC
	LDA #$ff		; 3 seconds
	STA Xretry2		;
	JSR GetByte		;
	BCC Wait4CRC	; wait for something to come in...
	CMP #'C			; is it the "C" to start a CRC xfer?
	BEQ SetCRC		; yes
	CMP #NAK
	BNE Wait1
	LDA #$C0
	.BYTE $2C
SetCRC
	LDA #$80
	STA USEXMOD		;receiver wants std xmodem
	JMP SetstAddr
Wait1
	CMP #CAN
	BEQ *+6
	CMP #ESC		; is it a cancel? <Esc> Key
	BNE Wait4CRC	; No, wait for another character
	JMP PrtAbort	; Print abort msg and exit
SetstAddr
	LDY #$00		; init data block offset to 0
	LDA #$01		; manually load Blk #1
	STA Rbuff		; into 1st byte
	LDA #$FE		; load 1's comp of block #
	STA Rbuff+1		; into 2nd byte
	;when transferring disk images, we don't send load address
	;LDX #$04		; preload X to Receive buffer
	;LDA TDATA		; load low byte of start address
	;STA Rbuff+2	; into 3rd byte
	;LDA TDATA+1	; load hi byte of start address
	;STA Rbuff+3	; into 4th byte
	LDX #$02        ;our buffer only has 2 byte preloaded now
	JMP Ldbuff1		; jump into buffer load routine

LdBuffer
	LDA lastblk		; Was the last block sent?
	BEQ LdBuff0		; no, send the next one
	JMP Done		; yes, we're done
LdBuff0
	LDX #$02		; init pointers
	LDY #$00		;
	INC Blkno		; inc block counter
	LDA Blkno		;
	STA Rbuff		; save in 1st byte of buffer
	EOR #$FF		;
	STA Rbuff+1		; save 1's comp of blkno next

LdBuff1
	LDA (TDATA),y		; save 128 bytes of data
	STA Rbuff,x		;
LdBuff2 
	LDA TDATA+1 	;
	CMP EOFPH		;
	BCC LdBuff4		;
	LDA TDATA		;
	CMP EOFP		; Are we at the last address?
	BCC LdBuff4		; no, inc pointer and continue
	    
	BIT lastdata	; set when last transfer block
	BMI LdBuf2b
	STX Xsavem      ;save output buffer pos for next time we are called
	CLC
	RTS
LdBuf2b                 ;no more blocks to send ; if we're at the start of a block finish
	CPX #$02        ;otherwise continue to pad to end
	BNE LdBuf2a
LdBuf2c
	LDA #EOT
	JSR Put_Chr
	JSR GetByte
	CMP #ACK
	BEQ LdBuf2d
	CMP #NAK
	BEQ LdBuf2c
	CMP #CAN
	BEQ LdBuf2d
	CMP #ESC
	BNE LdBuf2c
LdBuf2d
	JMP Done
LdBuf2a
	INC lastblk		; Yes, Set last byte flag
LdBuff3
	INX
	CPX #$82		; Are we at the end of the 128 byte block?
	BEQ DoCRC		; Yes, calc CRC
	LDA #$00		; Fill rest of 128 bytes with $00
	STA Rbuff,x		;
	BEQ LdBuff3		; Branch always

LdBuff4
	INC TDATA		; Inc address pointer
	BNE LdBuff5		;
	INC TDATA+1		;
LdBuff5
	INX
	CPX #$82		; last byte in block?
	BNE LdBuff1		; no, get the next

DoCRC
	LDY #$82
	JSR CalcCRC
	LDA crch		; save Hi byte of CRC to buffer
	STA Rbuff,y		;
	INY 		;
	LDA crc			; save lo byte of CRC to buffer
	STA Rbuff,y		;
Resend
	LDX #$00		;
	LDA #SOH		; Send start block command
	JSR Put_Chr		;
SendBlk
	LDA Rbuff,x		; Send 133 bytes in buffer to the console
	JSR Put_Chr
	INX
	CPX #$84
	BCC SendBlk
	BIT USEXMOD		;using CRC mode?
	BVS sendbl2		;no checksum, so branch
sendbl1
	CPX #$84		; last byte?
	BNE SendBlk		; no, get next
sendbl2
	LDA #$FF		; yes, set 3 second delay
	STA Xretry2		; and
	JSR GetByte		; Wait for Ack/Nack
	BCC Seterror	; No chr received after 3 seconds, resend
	CMP #NAK		;
	BEQ Seterror	; NAK, inc errors and resend
	CMP #ESC		;
	BEQ PrtAbort	; Esc pressed to abort
	CMP #ACK		; Chr received... is it:
	BNE SetError
	LDA #$00		;reset error count
	STA Xerrcnt
	JMP LdBuffer	; ACK, send next block
Seterror
	INC Xerrcnt		; Inc error counter
	LDA Xerrcnt		;
	CMP #$0A		; are there 10 errors in a row? (Xmodem spec for failure)
	BCC Resend		; no, resend block
	JSR PRINT
	.BYTE CAN,CAN,CAN,'Errors Exceeded!',CR,LF,0
	SEC
	RTS
PrtAbort
	JSR Flush		; yes, too many errors, flush buffer,
	JSR PRINT
	.BYTE CAN,CAN,CAN,'Abort Received!',CR,LF,0
	SEC
	RTS
Done
	JMP Print_Good	; All Done..Print msg and exit


Print_Err
	LDX #$00		; PRINT Error message
PrtErr1
	LDA ErrMsg,x
	BEQ PrtErr2
	JSR Put_Chr
	INX
	BNE PrtErr1
PrtErr2
	RTS
ErrMsg
	.BYTE CAN,CAN,CAN, "Transfer Error!",$A,$D,0

GotAbort
	JSR PrtAbort
	LDA #ABORTRECV	; Error code in "A" of desired
	SEC
	RTS
;
;
;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
;
; subroutines
;
;
GetByte
	LDA #$00		; wait for chr input and cycle timing loop
	STA	Xretry		; set low value of timing loop
StartCrcLp
	JSR Get_chr 	; get chr from serial port, don't wait
	BCS GetByte1	; got one, so exit
	EOR $FF,X       ; waste some time
	EOR $FF,X
	DEC Xretry		; no character received, so dec counter
	BNE StartCrcLp	;
	DEC Xretry2		; dec hi byte of counter
	BNE StartCrcLp	; look for character again
	CLC 		    ; if loop times out, CLC, else SEC and return
GetByte1
	RTS			    ; with character in "A"
;
Flushm    			;flush lite
	LDA #$20
	.BYTE $2c   	;bit
Flush
	LDA #$70		; flush receive buffer
	STA Xretry2		; flush until empty for ~1 sec.
Flush1
	JSR GetByte		; read the port
	BCS Flush		; if chr recvd, wait for another
	RTS 			; else done
;
FlushQ
	JSR Get_chr
	BCC FlushQ1
	CMP #NAK
	BEQ FlushQ
	RTS
FlushQ1
	LDA #$00
	RTS

PrintMsg
	JSR PRINT
	.BYTE 'Begin XMODEM/',0
	BIT USEXMOD
	BVS Printm1
	JSR PRINT
	.BYTE 'CRC',0
	BCC Printm2
Printm1
	JSR PRINT
	.BYTE 'Checksum',0
Printm2
	JSR PRINT
	.BYTE ' transfer.  Press <ESC> to abort...'
	.BYTE CR, LF, 0
	RTS

Print_Good
	JSR PRINT
	.BYTE EOT,EOT,EOT,CR,LF,CR,LF   ; send end of file three times
	.BYTE 'Transfer Complete!'
	.BYTE CR, LF, 0
	CLC
	RTS

CalcChk
	LDY #$02
	LDA #$00
CalcCh1
	CLC
	ADC	Rbuff,y
	iny
	cpy	#$82		; done yet?
	bne	CalcCh1
	STA crch		;value of checksum byte use when creating send block
	RTS
;=========================================================================
;
;
;  CRC subroutines 
;
;
CalcCRC
	BIT USEXMOD
	BVS CalcChk
	LDA #$00		; yes, calculate the CRC for the 128 bytes
	STA crc
	STA crch
	LDY #$02
CalcCRC1
	LDA Rbuff,y
	JSR UpdCRC
	INY
	CPY #$82		; done yet?
	BNE CalcCRC1	; no, get next
	RTS

UpdCrc	
	EOR crc+1 		; Quick CRC computation with lookup tables
	TAX 			; updates the two bytes at crc & crc+1
	LDA crc			; with the byte send in the "A" register
	EOR CRCHI,X
	STA crc+1
	LDA CRCLO,X
	STA crc
	RTS
;
;
; Alternate solution is to build the two lookup tables at run-time.  This might
; be desirable if the program is running from ram to reduce binary upload time.
; The following code generates the data for the lookup tables.  You would need to
; un-comment the variable declarations for crclo & crchi in the Tables and Constants
; section above and call this routine to build the tables before calling the
; "xmodem" routine.
;
;MAKECRCTABLE
;		ldx 	#$00
;		LDA	#$00
;zeroloop	sta 	crclo,x
;		sta 	crchi,x
;		inx
;		bne	zeroloop
;		ldx	#$00
;fetch		txa
;		eor	crchi,x
;		sta	crchi,x
;		ldy	#$08
;fetch1		asl	crclo,x
;		rol	crchi,x
;		bcc	fetch2
;		lda	crchi,x
;		eor	#$10
;		sta	crchi,x
;		lda	crclo,x
;		eor	#$21
;		sta	crclo,x
;fetch2		dey
;		bne	fetch1
;		inx
;		bne	fetch
;		rts
;
; The following tables are used to calculate the CRC for the 128 bytes
; in the xmodem data blocks.  You can use these tables if you plan to
; store this program in ROM.  If you choose to build them at run-time,
; then just delete them and define the two labels: crclo & crchi.
;
; low byte CRC lookup table (should be page aligned)

*= *+255/256*256 ;page align the following tables
crclo
 .byte $00,$21,$42,$63,$84,$A5,$C6,$E7,$08,$29,$4A,$6B,$8C,$AD,$CE,$EF
 .byte $31,$10,$73,$52,$B5,$94,$F7,$D6,$39,$18,$7B,$5A,$BD,$9C,$FF,$DE
 .byte $62,$43,$20,$01,$E6,$C7,$A4,$85,$6A,$4B,$28,$09,$EE,$CF,$AC,$8D
 .byte $53,$72,$11,$30,$D7,$F6,$95,$B4,$5B,$7A,$19,$38,$DF,$FE,$9D,$BC
 .byte $C4,$E5,$86,$A7,$40,$61,$02,$23,$CC,$ED,$8E,$AF,$48,$69,$0A,$2B
 .byte $F5,$D4,$B7,$96,$71,$50,$33,$12,$FD,$DC,$BF,$9E,$79,$58,$3B,$1A
 .byte $A6,$87,$E4,$C5,$22,$03,$60,$41,$AE,$8F,$EC,$CD,$2A,$0B,$68,$49
 .byte $97,$B6,$D5,$F4,$13,$32,$51,$70,$9F,$BE,$DD,$FC,$1B,$3A,$59,$78
 .byte $88,$A9,$CA,$EB,$0C,$2D,$4E,$6F,$80,$A1,$C2,$E3,$04,$25,$46,$67
 .byte $B9,$98,$FB,$DA,$3D,$1C,$7F,$5E,$B1,$90,$F3,$D2,$35,$14,$77,$56
 .byte $EA,$CB,$A8,$89,$6E,$4F,$2C,$0D,$E2,$C3,$A0,$81,$66,$47,$24,$05
 .byte $DB,$FA,$99,$B8,$5F,$7E,$1D,$3C,$D3,$F2,$91,$B0,$57,$76,$15,$34
 .byte $4C,$6D,$0E,$2F,$C8,$E9,$8A,$AB,$44,$65,$06,$27,$C0,$E1,$82,$A3
 .byte $7D,$5C,$3F,$1E,$F9,$D8,$BB,$9A,$75,$54,$37,$16,$F1,$D0,$B3,$92
 .byte $2E,$0F,$6C,$4D,$AA,$8B,$E8,$C9,$26,$07,$64,$45,$A2,$83,$E0,$C1
 .byte $1F,$3E,$5D,$7C,$9B,$BA,$D9,$F8,$17,$36,$55,$74,$93,$B2,$D1,$F0

; hi byte CRC lookup table (should be page aligned)
;		*= $7d00
crchi
 .byte $00,$10,$20,$30,$40,$50,$60,$70,$81,$91,$A1,$B1,$C1,$D1,$E1,$F1
 .byte $12,$02,$32,$22,$52,$42,$72,$62,$93,$83,$B3,$A3,$D3,$C3,$F3,$E3
 .byte $24,$34,$04,$14,$64,$74,$44,$54,$A5,$B5,$85,$95,$E5,$F5,$C5,$D5
 .byte $36,$26,$16,$06,$76,$66,$56,$46,$B7,$A7,$97,$87,$F7,$E7,$D7,$C7
 .byte $48,$58,$68,$78,$08,$18,$28,$38,$C9,$D9,$E9,$F9,$89,$99,$A9,$B9
 .byte $5A,$4A,$7A,$6A,$1A,$0A,$3A,$2A,$DB,$CB,$FB,$EB,$9B,$8B,$BB,$AB
 .byte $6C,$7C,$4C,$5C,$2C,$3C,$0C,$1C,$ED,$FD,$CD,$DD,$AD,$BD,$8D,$9D
 .byte $7E,$6E,$5E,$4E,$3E,$2E,$1E,$0E,$FF,$EF,$DF,$CF,$BF,$AF,$9F,$8F
 .byte $91,$81,$B1,$A1,$D1,$C1,$F1,$E1,$10,$00,$30,$20,$50,$40,$70,$60
 .byte $83,$93,$A3,$B3,$C3,$D3,$E3,$F3,$02,$12,$22,$32,$42,$52,$62,$72
 .byte $B5,$A5,$95,$85,$F5,$E5,$D5,$C5,$34,$24,$14,$04,$74,$64,$54,$44
 .byte $A7,$B7,$87,$97,$E7,$F7,$C7,$D7,$26,$36,$06,$16,$66,$76,$46,$56
 .byte $D9,$C9,$F9,$E9,$99,$89,$B9,$A9,$58,$48,$78,$68,$18,$08,$38,$28
 .byte $CB,$DB,$EB,$FB,$8B,$9B,$AB,$BB,$4A,$5A,$6A,$7A,$0A,$1A,$2A,$3A
 .byte $FD,$ED,$DD,$CD,$BD,$AD,$9D,$8D,$7C,$6C,$5C,$4C,$3C,$2C,$1C,$0C
 .byte $EF,$FF,$CF,$DF,$AF,$BF,$8F,$9F,$6E,$7E,$4E,$5E,$2E,$3E,$0E,$1E
;
;
; End of Xmodem
;
BUFFER	=	*
*=*+$1000 ;space for track buffer

.END

